function A = anyv(B)
% ANYV Vector-version of ANY computed over all dimensions
%
% A = ANYV(B)
  

% Jerod Weinman 
% jerod@acm.org
% (c) 2008

% $Id: anyv.m 1 2009-05-07 13:41:00Z weinman $
 
% Jerod Weinman
% jerod@acm.org
% /home/data/svn/vision/tools/package/GPL_STATEMENT
 
  
  A = any(B(:));
  
