function Y = sumv(X)
% SUMV Sum of ALL elements in X
%
%    Y = SUMV(X)
    
% Jerod Weinman 
% jerod@acm.org
    
% $Id: sumv.m 1 2009-05-07 13:41:00Z weinman $
 
% Jerod Weinman
% jerod@acm.org
% /home/data/svn/vision/tools/package/GPL_STATEMENT
 
    

  Y = sum(X(:));
