function Y = mkvector(X)
% MKVECTOR Reshape an N-tensor into a 1-tensor (vector)
%
%    Y = MKVECTOR(X)
%
% See also MKRVECTOR, MKCVECTOR
    
      
% Jerod Weinman 
% jerod@acm.org

% $Id: mkvector.m 1 2009-05-07 13:41:00Z weinman $
 
% Jerod Weinman
% jerod@acm.org
% /home/data/svn/vision/tools/package/GPL_STATEMENT
 

  Y = X(:);
  
